/**
 * Initialize the SDK with the specified configuration
 */
function initSdk(config, gvl, sdkURL) {
  if (config) {
    // Set the global window configuration to the config object
    window.didomiConfig = prepareConfigFromMobile(config);

    // If app, vendors, and iab exist, set the vendorList property
    if (
      window.didomiConfig.app &&
      window.didomiConfig.app.vendors &&
      window.didomiConfig.app.vendors.iab
    ) {
      window.didomiConfig.app.vendors.iab.vendorList =
        prepareGVLFromMobile(gvl);
    }
  }

  var script = document.createElement("script");
  script.setAttribute("type", "text/javascript");
  script.setAttribute("src", sdkURL);

  document.getElementsByTagName("head")[0].appendChild(script);
}

/**
 * Open the notice screen
 */
function openNotice(options) {
  window.didomiOnReady = window.didomiOnReady || [];
  window.didomiOnReady.push(function (Didomi) {
    if (options != null && options.deepLinkView != null) {
      Didomi.preferences.show(
        options.deepLinkView == 0 ? "purposes" : "vendors"
      );
    } else {
      Didomi.notice.show();
    }
  });

  window.didomiEventListeners = window.didomiEventListeners || [];
  window.didomiEventListeners.push(
    {
      event: "consent.changed",
      listener: function (event) {
        // If the change is coming from the WebView we don't want to trigger the message back to the WebView.
        if (event.action == "webview") {
          return;
        }
        androidInterface.onConsentChanged(JSON.stringify(Didomi.getUserStatus()));
      },
    },
    {
      event: 'api.error',
      listener: function ({ id, reason }) {
        androidInterface.onError(id, reason);
      }
    },
    {
      event: 'notice.clickagree',
      listener: function () {
        androidInterface.onNoticeClickAgree();
      }
    },
    {
      event: 'notice.clickdisagree',
      listener: function () {
        androidInterface.onNoticeClickDisagree();
      }
    },
    {
      event: 'notice.clickmoreinfo',
      listener: function () {
        androidInterface.onNoticeClickMoreInfo();
      }
    },
    {
      event: 'notice.clickviewvendors',
      listener: function () {
        androidInterface.onNoticeClickViewVendors();
      }
    },
    {
      event: 'notice.hidden',
      listener: function () {
        androidInterface.onNoticeHidden();
      }
    },
    {
      event: 'notice.shown',
      listener: function () {
        androidInterface.onNoticeShown();
      }
    },
    {
      event: 'preferences.clickagreetoall',
      listener: function () {
        androidInterface.onPreferencesClickAgreeToAll();
      }
    },
    {
      event: 'preferences.clickcategoryagree',
      listener: function ({ categoryId }) {
        androidInterface.onPreferencesClickCategoryAgree(categoryId);
      }
    },
    {
      event: 'preferences.clickcategorydisagree',
      listener: function ({ categoryId }) {
        androidInterface.onPreferencesClickCategoryDisagree(categoryId);
      }
    },
    {
      event: "preferences.clickclose",
      listener: function () {
        // Not handled by mobile
      }
    },
    {
      event: 'preferences.clickdisagreetoall',
      listener: function () {
        androidInterface.onPreferencesClickDisagreeToAll();
      }
    },
    {
      event: 'preferences.clickpurposeagree',
      listener: function ({ purposeId }) {
        androidInterface.onPreferencesClickPurposeAgree(purposeId);
      }
    },
    {
      event: 'preferences.clickpurposedisagree',
      listener: function ({ purposeId }) {
        androidInterface.onPreferencesClickPurposeDisagree(purposeId);
      }
    },
    {
      event: 'preferences.clicksavechoices',
      listener: function () {
        androidInterface.onPreferencesClickSaveChoices(purposeId);
      }
    },
    {
      event: 'preferences.clickvendoragree',
      listener: function ({ vendorId }) {
        androidInterface.onPreferencesClickVendorAgree(vendorId);
      }
    },
    {
      event: 'preferences.clickvendordisagree',
      listener: function ({ vendorId }) {
        androidInterface.onPreferencesClickVendorDisagree(vendorId);
      }
    },
    {
      event: 'preferences.clickvendorsavechoices',
      listener: function () {
        androidInterface.onPreferencesClickVendorSaveChoices();
      }
    },
    {
      event: 'preferences.clickviewvendors',
      listener: function () {
        androidInterface.onPreferencesClickViewVendors();
      }
    },
    {
      event: "preferences.hidden",
      listener: function () {
        androidInterface.onPreferencesHidden();
      }
    },
    {
      event: "preferences.shown",
      listener: function () {
        androidInterface.onPreferencesShown();
      }
    }
  );
}

/**
 * Function used to convert objects that are mapped by ID into an array of objects that only contain IDs.
 * @param {*} object JSON object.
 * @returns array of objects that only contain an ID.
 */
function convertObjectToArrayWithIDs(object) {
  return Object.keys(object).map(function (id) {
    return {
      id,
    };
  });
}

/**
 * Prepare config file to be consumed by the Web SDK, by disabling the features already handled by Mobile SDKs.
 * @param {*} configFromMobile Config provided by mobile.
 * @returns Config with unneeded features disabled
 */
function prepareConfigFromMobile(configFromMobile) {
  if (configFromMobile == null) {
    return null;
  }

  configFromMobile.events = configFromMobile.events ?? {}
  configFromMobile.events.enabled = false;
  
  if (configFromMobile.sync) {
    configFromMobile.sync.enabled = false;
  }
  
  if (configFromMobile.app && configFromMobile.app.consentString) {
    configFromMobile.app.consentString.signatureEnabled = false;
  }

  return configFromMobile;
}

/**
 * Prepare GVL to be consumed by the Web SDK based on the format used by Mobile SDKs.
 * @param {*} gvlFromMobile GVL in the format used by mobile.
 * @returns GVL in the format used by web.
 */
function prepareGVLFromMobile(gvlFromMobile) {
  if (gvlFromMobile == null) {
    return null;
  }

  gvlFromMobile.purposes = convertObjectToArrayWithIDs(gvlFromMobile.purposes);
  gvlFromMobile.specialPurposes = convertObjectToArrayWithIDs(
    gvlFromMobile.specialPurposes
  );
  gvlFromMobile.features = convertObjectToArrayWithIDs(gvlFromMobile.features);
  gvlFromMobile.specialFeatures = convertObjectToArrayWithIDs(
    gvlFromMobile.specialFeatures
  );
  gvlFromMobile.dataCategories = convertObjectToArrayWithIDs(
    gvlFromMobile.dataCategories
  );

  gvlFromMobile.stacks = Object.keys(gvlFromMobile.stacks).map(function (key) {
    var stack = gvlFromMobile.stacks[key];
    return {
      id: stack.id,
      purposeIds: stack.purposes || [],
      specialFeatureIds: stack.specialFeatures || [],
    };
  });

  gvlFromMobile.vendors = Object.keys(gvlFromMobile.vendors).map(function (
    key
  ) {
    var vendor = gvlFromMobile.vendors[key];
    return {
      ...vendor,
      purposeIds: vendor.purposes || [],
      flexiblePurposeIds: vendor.flexiblePurposes || [],
      specialPurposeIds: vendor.specialPurposes || [],
      legIntPurposeIds: vendor.legIntPurposes || [],
      featureIds: vendor.features || [],
      specialFeatureIds: vendor.specialFeatures || [],
    };
  });
  return gvlFromMobile;
}
